import { _decorator, Component, Node, Vec3, Quat, assetManager, AssetManager, Asset, director, instantiate, game, AudioSourceComponent, AudioClip, Sprite, Texture2D, SpriteFrame, loader, Size, UITransform, Button, EventHandler, MeshRenderer, geometry, Vec2, Camera, RenderTexture, gfx, view } from 'cc'

const { ccclass, property } = _decorator


@ccclass('Tools')
export class Tools
{
    public static async loadBundle(bundleName:string)
    {
        return new Promise((res,rej)=>
        {
            assetManager.loadBundle(bundleName,function(err,bundle)
            {
                if (!err) { res(bundle) } else { rej(err) }
            })
        })
    }
    public static async loadAssetByBundle(bundle:AssetManager.Bundle|any,path:string)
    {
        return new Promise((res,rej)=>
        {
            bundle.load(path,function(err:any,data:Asset)
            {
                if (!err) { res(data) } else { rej(err) }
            })
        })
    }

    private static bgm:Node = null;
    private static sound:Node = null;
    private static soundBundle:AssetManager.Bundle = null;
    public static async playBGM(name:string)
    {
        if (!Tools.bgm)
        {
            Tools.bgm = new Node()
            game.addPersistRootNode(Tools.bgm)
            Tools.bgm.addComponent(AudioSourceComponent)
        }

        let com = Tools.bgm.getComponent(AudioSourceComponent)
        com.playOnAwake = false;

        if (!Tools.soundBundle)
        {
            Tools.soundBundle = await Tools.loadBundle('BundleSound') as AssetManager.Bundle;
        }

        let sound = await Tools.loadAssetByBundle(Tools.soundBundle,name) as AudioClip;
        com.clip = sound;
        com.loop = true;
        com.play();
    }
    
    public static async playSound(name:string,loop:boolean=false)
    {
        if (!Tools.sound)
        {
            Tools.sound = new Node()
            game.addPersistRootNode(Tools.sound)
            let c = Tools.sound.addComponent(AudioSourceComponent)
            c.playOnAwake = false;
        }
        let com = Tools.sound.getComponent(AudioSourceComponent)
        if (!Tools.soundBundle)
        {
            Tools.soundBundle = await Tools.loadBundle('BundleSound') as AssetManager.Bundle;
        }
        let sound = await Tools.loadAssetByBundle(Tools.soundBundle,name) as AudioClip;
        com.clip = sound;
        com.loop = loop;
        com.play();
    }
}

